library out_minispectrum;

(* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version 
 * 1.1 (the "License"); you may not use this file except in compliance with 
 * the License. You may obtain a copy of the License at 
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the MiniSpectrum Output Module for Lightning.
 *
 * The Initial Developer of the Original Code is
 * Tomsoft.
 * Portions created by Tomsoft are Copyright (C) 2009
 * Tomsoft. All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Sools <tom@sools.com>
 *
 * ***** END LICENSE BLOCK ***** *)

{$IMAGEBASE $41210000}

uses
  Classes,
  SysUtils,
  Windows,
  Math,
  StdCtrls,
  ExtCtrls,
  Controls,
  Buttons,
  Graphics,
  common in '..\common.pas';

{$R *.RES}

const
  spectrumWidth = 128;
  spectrumHeight = 64;
  spectrumColor = clGreen;
  backgroundColor = clBtnFace;
  markerColor = clRed;

type
  TDetails = class(TObject)
  public
    marker:       TLevel;
    pSpectrum:    TPanel;
    pbSpectrum:   TPaintBox;
    spectrum:     TSpectrum;
    bitmap:       TBitmap;
    bitmapFilled: Boolean;
    procedure pbSpectrumPaint(Sender: TObject);
  end;

procedure TDetails.pbSpectrumPaint(Sender: TObject);
begin
  if bitmapFilled then
    pbSpectrum.Canvas.Draw(0, 0, bitmap);
end;

function init(name: PChar; nameLength: DWORD): DWORD; cdecl;
begin
  Result := return('OMiniSpectrum|Visual', name, nameLength);
end;

procedure finish(); cdecl;
begin
  //
end;

function create(controlHandle: THandle): PLightningModule; cdecl;
var
  myDetails: TDetails;
begin
  New(Result);
  with Result^ do
  begin
    handle        := controlHandle;
    title         := '           Spectrum';
    settings      := False;
    width         := spectrumWidth + 5;
    height        := 8 + 24 + spectrumHeight + 3;
    inputTop      := -1;
    infoTop       := 8;
    outputTop     := -1;
    inputs        := 2;
    outputs       := 0;
    GetMem(inputTooltip, maxLength);
    StrCopy(inputTooltip, 'Spectrum|Marker value');
    outputTooltip := '';
    myDetails     := TDetails.Create;
    details       := myDetails;
    reserved      := nil;
  end;
  with myDetails do
  begin
    bitmapFilled := False;
    // bitmap for buffering of graphics
    bitmap := TBitmap.Create();
    with bitmap do
    begin
      Width  := spectrumWidth;
      Height := spectrumHeight;
    end;
    pSpectrum := TPanel.Create(nil);
    with pSpectrum do
    begin
      Top        := 32;
      Left       := 2;
      Width      := spectrumWidth;
      Height     := spectrumHeight;
      BevelOuter := bvNone;
    end;
    pbSpectrum := TPaintBox.Create(pSpectrum);
    with pbSpectrum do
    begin
      ControlStyle := ControlStyle + [csOpaque];
      Top          := 0;
      Left         := 0;
      Width        := spectrumWidth;
      Height       := spectrumHeight;
      OnPaint      := pbSpectrumPaint;
      Parent       := pSpectrum;
    end;
    pSpectrum.ParentWindow := controlHandle;
    marker := levelMin;
  end;
end;

procedure destroy(lightningModule: PLightningModule); cdecl;
begin
  with TDetails(lightningModule^.details) do
  begin
    pbSpectrum.Free();
    pSpectrum.Free();
    bitmap.Free();
    Free();
  end;
  FreeMem(lightningModule^.inputTooltip);
  Dispose(lightningModule);
end;

function  action(lightningModule: PLightningModule; input, output: Integer; inputConnected, outputConnected: PChar; var bridge: TSpectrum): Boolean; cdecl;
var
  details: TDetails;
  bString: string;
  i, index, fftPerPixel, levelPerPixel, lineTop: Integer;
  value:  TLevel;
begin
  Result := False;
  details := TDetails(lightningModule^.details);
  bString := inputConnected;
  parse(bString, '|');
  if parse(bString, '|') = '' then
    details.marker := levelMin;
  if input = 1 then
    details.spectrum := bridge
  else // if input = 2 then
    details.marker := spectrumAverage(bridge, fftMin, fftMax);
  if details.bitmap <> nil then
  begin
    fftPerPixel := Ceil((fftMax - fftMin + 1) / spectrumWidth);
    levelPerPixel := Ceil((levelMax - levelMin + 1) / spectrumHeight);
    details.bitmap.Width              := spectrumWidth;
    details.bitmap.Height             := spectrumHeight;
    details.bitmap.Canvas.Pen.Color   := backgroundColor;
    details.bitmap.Canvas.Brush.Color := backgroundColor;
    details.bitmap.Canvas.FillRect(Rect(0, 0, spectrumWidth, spectrumHeight));
    for i := 0 to spectrumWidth - 1 do
    begin
      index := fftMin + (i * fftPerPixel);
      if index + fftPerPixel - 1 <= fftMax - fftMin + 1 then
        value := spectrumAverage(details.spectrum, index, index + fftPerPixel - 1)
      else
        value := 0;
      if value > levelMin then
      begin
        details.bitmap.Canvas.Pen.Color := spectrumColor;
        details.bitmap.Canvas.MoveTo(i, spectrumHeight - Trunc(value / levelPerPixel));
        details.bitmap.Canvas.LineTo(i, spectrumHeight);
      end;
    end;
    if details.marker > levelMin then
    begin
      details.bitmap.Canvas.Pen.Color := markerColor;
      lineTop := spectrumHeight - Trunc(details.marker / levelPerPixel);
      details.bitmap.Canvas.MoveTo(0, lineTop);
      details.bitmap.Canvas.LineTo(Trunc((fftMax - fftMin + 1) / fftPerPixel), lineTop);
    end;
    details.bitmapFilled := True;
    details.pbSpectrum.OnPaint(details);
  end;
end;

procedure show(lightningModule: PLightningModule; handle: THandle); cdecl;
begin
  //
end;

procedure hide(lightningModule: PLightningModule); cdecl;
begin
  //
end;

procedure open(lightningModule: PLightningModule; settings: PChar); cdecl;
begin
  //
end;

function  save(lightningModule: PLightningModule; settings: PChar; settingsLength: DWORD): DWORD; cdecl;
begin
  Result := return('', settings, settingsLength);
end;

exports
  init, finish, create, destroy, action, show, hide, open, save;

end.
